@@ -21,7 +21,7 @@ import itertools
 import collections
 
 __author__  = 'Miguel Turner'
-__version__ = '0.2.9'
+__version__ = '0.3.0'
 __license__ = 'MIT'
 
 __all__ = ['Config', 'IniFormat', 'ConfigError', 'Coercer', 'CoerceError']
@@ -47,10 +47,8 @@ class ConfigSection(collections.MutableMapping):
     def __init__(self, name, parent):
         self._name = name
         self._value = NoValue
-        self._cache = NoValue
         self._default = NoValue
         self._type = None
-        self._has_type = False
         self._parent = parent
         
         self.comment = None
@@ -115,14 +113,14 @@ class ConfigSection(collections.MutableMapping):
 
         If *sources* are provided, syncs only those sources. Otherwise,
         syncs the sources in :attr:`~config.Config.sources`.
+        
+        *format* can be used to override the format used to read/write from
+        the sources.
         """
-        
         format = kwargs.pop('format', None)
-        
-        # if caching, adapt cached values
-        if self.cache_values:
-            for section in self.sections(recurse=True):
-                section._adapt_cache()
+        if kwargs:
+            err = "sync() got an unexpected keyword argument '{}'"
+            raise TypeError(err.format(kwargs.popitem()[0]))
         
         sources, format = self._process_sources(sources, format)
         
@@ -163,9 +161,7 @@ class ConfigSection(collections.MutableMapping):
         file in a manner consistent with the active :class:`~profig.Format`.
         """
         section = self._create_section(key)
-        section._cache = NoValue
         section._type = type or default.__class__
-        section._has_type = True
         section.set_default(default)
         section.comment = comment
     
@@ -302,40 +298,35 @@ class ConfigSection(collections.MutableMapping):
                 section._reset()
     
     def value(self, convert=True, type=None):
-        """
-        Get the section's value.
-        To get the underlying string value, set *convert* to `False`.
+        """Get the section's value.
+        
+        To get the internal string value, set *convert* to `False`.
         If *convert* is `False`, *type* will be ignored.
         """
         if not convert:
             if self._value is not NoValue:
                 return self._value
-        elif type is None and self._cache is not NoValue:
-            return self._cache
         elif self._value is not NoValue:
-            value = self.convert(self._value, type)
-            self._cache = self._cache_value(value)
-            return value
+            return self.convert(self._value, type)
         
         return self.default(convert, type)
     
-    def set_value(self, value):
-        """Set the section's value."""
-        if not isinstance(value, str):
-            strvalue = self.adapt(value)
-            if strvalue != self._value:
-                self._value = strvalue
-                self._cache = self._cache_value(value)
-                self.dirty = True
+    def set_value(self, value, adapt=True):
+        """Set the section's value.
         
-        elif value != self._value:
+        To set the internal string value, set *adapt* to `False`.
+        """
+        if adapt:
+            value = self.adapt(value)
+        elif isinstance(value, bytes) and self._type is not bytes:
+            value = value.decode(self._root.encoding)
+        if value != self._value:
             self._value = value
-            self._cache = NoValue
             self.dirty = True
     
     def default(self, convert=True, type=None):
-        """
-        Get the section's default value.
+        """Get the section's default value.
+        
         To get the underlying string value, set *convert* to `False`.
         If *convert* is `False`, *type* will be ignored.
         """
@@ -343,32 +334,20 @@ class ConfigSection(collections.MutableMapping):
             if self._default is not NoValue:
                 return self._default
         elif self._default is not NoValue:
-            if type is None and self._value is NoValue and self._cache is not NoValue:
-                # only use cache if self._value hasn't been set
-                return self._cache
-            else:
-                value = self.convert(self._default, type)
-                # only set cache if self._value hasn't been set
-                if self._value is NoValue:
-                    self._cache = self._cache_value(value)
-                return value
+            return self.convert(self._default, type)
         
         raise NoValueError(self.key)
     
-    def set_default(self, default):
+    def set_default(self, value, adapt=True):
+        """Set the section's default value.
+        
+        To set the internal string value, set *adapt* to `False`.
         """
-        Set the section's default value.
-        """
-        if not isinstance(default, str):
-            self._default = self.adapt(default)
-            # only set cache if self._value hasn't been set
-            if self._value is NoValue:
-                self._cache = self._cache_value(default)
-        else:
-            self._default = default
-            # only clear cache if self._value hasn't been set
-            if self._value is NoValue:
-                self._cache = NoValue
+        if adapt:
+            value = self.adapt(value)
+        elif isinstance(value, bytes) and self._type is not bytes:
+            value = value.decode(self._root.encoding)
+        self._default = value
     
     def items(self, convert=True):
         """Returns a (key, value) iterator over the unprocessed values of
@@ -376,25 +355,26 @@ class ConfigSection(collections.MutableMapping):
         for key in self:
             yield (key, self.section(key).value(convert))
     
-    def adapt(self, value):
-        if self._root.coerce_values:
-            if not self._has_type:
-                self._type = value.__class__
-            return self._root.coercer.adapt(value, self._type)
-        else:
+    def adapt(self, value, type=None):
+        """
+        value -> str
+        """
+        if not self._root.coercer:
             return value
+        # only set the type if it has not already been set
+        if self._type is None:
+            self._type = type or value.__class__
+        type = type or self._type
+        return self._root.coercer.adapt(value, type)
     
-    def convert(self, value, type):
-        if self._root.coerce_values:
-            return self._root.coercer.convert(value, type or self._type)
-        else:
-            return value
-    
-    def clear_cache(self, recurse=False):
-        """Clears cached values for this section. If *recurse* is
-        `True`, clears the cache for child sections as well."""
-        for section in self.sections(recurse):
-            section._cache = NoValue
+    def convert(self, string, type=None):
+        """
+        str -> value
+        """
+        if not self._root.coercer:
+            return string
+        type = type or self._type
+        return self._root.coercer.convert(string, type)
     
     ## utilities ##
     
@@ -428,7 +408,7 @@ class ConfigSection(collections.MutableMapping):
         return write_lines
     
     def _write(self, source, format, lines=None):
-        file = format.open(source, 'w')
+        file = format.open(source, 'wb')
         try:
             format.write(file, lines)
         finally:
@@ -499,22 +479,9 @@ class ConfigSection(collections.MutableMapping):
     def _reset(self):
         if self._value is not NoValue:
             self._value = NoValue
-            self._cache = NoValue
             self.dirty = True
-            if self._default is NoValue:
-                self._type = None
-    
-    def _adapt_cache(self):
-        if self._cache is not NoValue:
-            strvalue = self.adapt(self._cache)
-            if strvalue != self.value(convert=False):
-                self.set_value(strvalue)
-                self.dirty = True
-    
-    def _cache_value(self, value):
-        # no point in caching a string
-        should_cache = self._root.cache_values and not isinstance(value, str)
-        return value if should_cache else NoValue
+        if self._default is NoValue:
+            self._type = None
     
     def _dump(self, indent=2): # pragma: no cover
         rootlen = len(self._make_key(self._key))
@@ -540,6 +507,10 @@ class Config(ConfigSection):
     The dict class used internally can be set using *dict_type*. By default
     an `OrderedDict` is used.
     
+    A :class:`~profig.Coercer` can be set using *coercer*. If no coercer is
+    passed in, a default will be created. If `None` is passed in, no coercer
+    will be set and values will be read from and written to sources directly.
+    
     This is a subclass of :class:`~profig.ConfigSection`.
     """
     
@@ -555,12 +526,13 @@ class Config(ConfigSection):
         format = kwargs.pop('format', 'ini')
         self.set_format(format)
         
-        self.coercer = Coercer()
-        register_booleans(self.coercer)
+        self.coercer = kwargs.pop('coercer', NoValue)
+        if self.coercer is NoValue:
+            self.coercer = Coercer(self.encoding)
+        if self.coercer:
+            register_booleans(self.coercer)
         
         self.sep = '.'
-        self.cache_values = True
-        self.coerce_values = True
     
     @classmethod
     def known_formats(cls):
@@ -620,7 +592,7 @@ class Format(BaseFormat):
         in a subclass."""
         raise NotImplementedError('abstract')
     
-    def open(self, source, mode='r', *args):
+    def open(self, source, mode='rb', *args):
         """Returns a file object.
         
         If *source* is a file object, returns *source*. If *mode* is 'w',
@@ -634,7 +606,7 @@ class Format(BaseFormat):
             if self.ensure_dirs is not None and 'w' in mode:
                 # ensure the path exists if any writing is to be done
                 ensure_dirs(os.path.dirname(source), self.ensure_dirs)
-            return io.open(source, mode, *args, encoding=self.encoding)
+            return io.open(source, mode, *args)
         else:
             source.seek(0)
             if 'w' in mode:
@@ -665,13 +637,14 @@ class Format(BaseFormat):
 
 class IniFormat(Format):
     name = 'ini'
-    delimeter = '='
-    comment_char = ';'
-    default_section = 'default'
-    _rx_section_header = re.compile('\[\s*(\S*)\s*\](\s*=\s*(\S*))?')
+    delimeter = b' = '
+    comment_char = b'; '
+    default_section = b'default'
+    _rx_section_header = re.compile(b'\[\s*(\S*)\s*\](\s*=\s*(\S*))?')
     
     def read(self, file):
         cfg = self.config
+        comment_char = self.comment_char.strip()
         section_name = self.default_section
         comment = None
         lines = []
@@ -691,9 +664,10 @@ class IniFormat(Format):
                 continue
             
             # comment line
-            if line.startswith(self.comment_char):
+            if line.startswith(comment_char):
                 flush_comment(lines, comment)
-                comment = Line(orgline, line.lstrip(self.comment_char).strip())
+                comment_text = line.lstrip(comment_char).strip().decode(self.encoding)
+                comment = Line(orgline, comment_text)
                 continue
             
             # section header
@@ -706,22 +680,25 @@ class IniFormat(Format):
                     section_name = self.default_section
                 
                 values[section_name] = value
-                comments[section_name] = comment.name if comment else None
+                if comment:
+                    comments[section_name] = comment.name
                 comment = None
                 
+                section_name = section_name.decode(self.encoding)
                 lines.append(Line(orgline, section_name, issection=True))
                 continue
             
             # must be a value
             try:
-                key, value = line.split(self.delimeter, 1)
+                key, value = line.split(self.delimeter.strip(), 1)
             except ValueError:
                 self._read_error(file, i, line)
                 continue
             
             key = cfg._keystr(cfg._make_key(section_name, key.strip()))
             values[key] = value.strip()
-            comments.setdefault(key, comment.name if comment else None)
+            if comment:
+                comments[key] = comment.name
             comment = None
             
             lines.append(Line(orgline, key, iskey=True))
@@ -734,33 +711,49 @@ class IniFormat(Format):
         for key, value in values.items():
             section = cfg.section(key)
             if not section.dirty and value is not None:
-                section.set_value(value)
+                section.set_value(value, adapt=False)
             if key in comments:
                 section.comment = comments[key]
         
         return lines
     
     def write_section(self, section, file, first=False):
+        write = lambda s: file.write(s.encode(self.encoding))
+        
         if not first and section.parent is section.root:
-            file.write('\n')
+            write('\n')
         
         if section.comment:
-            file.write('{} {}\n'.format(self.comment_char, section.comment))
+            file.write(self.comment_char)
+            write(section.comment)
+            write('\n')
         
         if section.parent is section.root:
             # header section
             if section.valid:
                 value = section.value(convert=False)
-                file.write('[{}] = {}\n'.format(section.name, value))
+                write('[{}]'.format(section.name))
+                file.write(self.delimeter)
+                if isinstance(value, bytes):
+                    file.write(value)
+                else:
+                    write(value)
+                write('\n')
             else:
-                file.write('[{}]\n'.format(section.name))
+                write('[{}]\n'.format(section.name))
+        
         elif section.valid:
             # value section
             cfg = self.config
             key = cfg._keystr(cfg._make_key(section.key)[1:])
             value = section.value(convert=False)
-            
-            file.write('{} {} {}\n'.format(key, self.delimeter, value))
+            write(key)
+            file.write(self.delimeter)
+            if isinstance(value, bytes):
+                file.write(value)
+            else:
+                write(value)
+            write('\n')
         
         section.dirty = False
     
@@ -799,7 +792,7 @@ class IniFormat(Format):
             else:
                 file.write(line.line)
         
-        # if there is a previous header section, write it's remaining values
+        # if there is an incomplete header section, write it's remaining values
         if header:
             for sec in header.sections(recurse=True):
                 if sec.key not in seen:
@@ -990,7 +983,7 @@ class Coercer:
         :meth:`~profig.Coercer.convert` for *type* will have to be one of the
         choices. *choices* must be a dict that maps converted->adapted
         representations."""
-        def verify(x, c=choices):
+        def verify(x, c):
             if x not in c:
                 err = "invalid choice {!r}, must be one of: {}"
                 raise ValueError(err.format(x, c))
@@ -1040,6 +1033,7 @@ class NotRegisteredError(CoerceError, KeyError):
 
 def register_default_coercers(coercer):
     """Registers adapters and converters for common types."""
+    import base64
     import binascii
     
     # None as the type does not change the value
@@ -1051,9 +1045,13 @@ def register_default_coercers(coercer):
     coercer.register(float, str, float)
     coercer.register(complex, str, complex)
     coercer.register(str, str, str)
-    coercer.register(bytes,
-        lambda x: binascii.hexlify(x).decode('ascii'),
-        lambda x: binascii.unhexlify(x.encode('ascii')))
+    coercer.register(bytes, bytes, bytes)
+    coercer.register('hex',
+        lambda x: binascii.hexlify(x),
+        lambda x: binascii.unhexlify(x))
+    coercer.register('base64',
+        lambda x: base64.b64encode(x),
+        lambda x: base64.b64decode(x))
     
     # collection coercers, simply comma delimited
     split = lambda x: x.split(',') if x else []
